# -*- coding: utf-8 -*- #
"""*********************************************************************************************"""
#   FileName     [ loss.py ]
#   Synopsis     [ the objective functions for speech separation ]
#   Source       [ Use some code from https://github.com/funcwj/uPIT-for-speech-separation and https://github.com/asteroid-team/asteroid ]
#   Author       [ Zili Huang ]
#   Copyright    [ Copyright(c), Johns Hopkins University ]
"""*********************************************************************************************"""

import torch
from itertools import permutations
import torch.nn.functional as F
from torch.nn.modules.loss import _Loss
from asteroid.losses import PITLossWrapper

device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")

class MSELoss(object):
    def __init__(self, num_srcs, mask_type):
        """
        Args:
            num_srcs (int):
                number of sources

            mask_type (str):
                type of mask to approach, currently supporting AM, PSM and
                NPSM. Please see Kolbæk M, Yu D, Tan Z H, et al
                Multitalker speech separation with utterance-level permutation
                invariant training of deep recurrent neural network
                for details
        """
        self.num_srcs = num_srcs
        self.mask_type = mask_type
        assert self.mask_type in ["AM", "PSM", "NPSM"]

    def compute_loss(self, masks, feat_length, source_attr, target_attr):
        feat_length = feat_length.to(device)
        mixture_spect = source_attr["magnitude"].to(device)
        targets_spect = [t.to(device) for t in target_attr["magnitude"]]
        mixture_phase = source_attr["phase"].to(device)
        targets_phase = [t.to(device) for t in target_attr["phase"]]

        def loss(permute):
            loss_for_permute = []
            for s, t in enumerate(permute):
                if self.mask_type == "AM":
                    refer_spect = targets_spect[t]
                elif self.mask_type == "PSM":
                    refer_spect = targets_spect[t] * torch.cos(mixture_phase - targets_phase[t])
                elif self.mask_type == "NPSM":
                    refer_spect = targets_spect[t] * F.relu(torch.cos(mixture_phase - targets_phase[t]))
                else:
                    raise ValueError("Mask type not defined.")

                utt_loss = torch.sum(
                    torch.sum(
                        torch.pow(masks[s] * mixture_spect - refer_spect, 2), -1),
                    -1)
                loss_for_permute.append(utt_loss)
            loss_perutt = sum(loss_for_permute) / feat_length
            return loss_perutt

        num_utts = feat_length.shape[0]
        pscore = torch.stack(
            [loss(p) for p in permutations(range(self.num_srcs))])
        min_perutt, _ = torch.min(pscore, dim=0)
        return torch.sum(min_perutt) / (self.num_srcs * num_utts)

class SISDRLoss(object):
    def __init__(self, num_srcs, n_fft, hop_length, win_length, window, center):
        self.num_srcs = num_srcs
        self.n_fft = n_fft
        self.hop_length = hop_length
        self.win_length = win_length
        if window == 'hann':
            self.window = torch.hann_window(win_length).cuda()
        self.center = center
        self.loss = PITLossWrapper(PairwiseNegSDR("sisdr"), pit_from="pw_mtx")

    def compute_loss(self, masks, feat_length, source_attr, wav_length, target_wav_list):
        mixture_stft = source_attr["stft"].to(device)
        bs = mixture_stft.size(0)
        est_targets = torch.zeros(bs, self.num_srcs, max(wav_length)).to(device)
        targets = torch.stack(target_wav_list, dim=1).to(device)
        for i in range(bs):
            mix_stft_utt = mixture_stft[i, :feat_length[i], :]
            est_src_list = []
            for j in range(self.num_srcs):
                mask_utt = masks[j][i, :feat_length[i], :]
                est_stft_utt = mix_stft_utt * mask_utt
                est_stft_utt = (torch.transpose(est_stft_utt, 0, 1)).unsqueeze(0)
                est_src = torch.istft(est_stft_utt, self.n_fft, hop_length=self.hop_length, win_length=self.win_length, window=self.window, center=self.center, length=wav_length[i])[0]
                if est_src.size(0) != wav_length[i]:
                    print("Warning: wav length doesn't match")
                    est_src = match_wave_length(est_src, wav_length[i])
                est_src_list.append(est_src)
            est_srcs = torch.stack(est_src_list, dim=0)
            est_targets[i, :, :wav_length[i]] = est_srcs
        loss = self.loss(est_targets, targets, length=wav_length)
        return loss


class PairwiseNegSDR(_Loss):
    r"""Base class for pairwise negative SI-SDR, SD-SDR and SNR on a batch.

    Args:
        sdr_type (str): choose between ``snr`` for plain SNR, ``sisdr`` for
            SI-SDR and ``sdsdr`` for SD-SDR [1].
        zero_mean (bool, optional): by default it zero mean the target
            and estimate before computing the loss.
        take_log (bool, optional): by default the log10 of sdr is returned.

    Shape:
        - est_targets : :math:`(batch, nsrc, ...)`.
        - targets: :math:`(batch, nsrc, ...)`.

    Returns:
        :class:`torch.Tensor`: with shape :math:`(batch, nsrc, nsrc)`. Pairwise losses.

    Examples
        >>> import torch
        >>> from asteroid.losses import PITLossWrapper
        >>> targets = torch.randn(10, 2, 32000)
        >>> est_targets = torch.randn(10, 2, 32000)
        >>> loss_func = PITLossWrapper(PairwiseNegSDR("sisdr"),
        >>>                            pit_from='pairwise')
        >>> loss = loss_func(est_targets, targets)

    References
        [1] Le Roux, Jonathan, et al. "SDR half-baked or well done." IEEE
        International Conference on Acoustics, Speech and Signal
        Processing (ICASSP) 2019.
    """

    def __init__(self, sdr_type, zero_mean=True, take_log=True, EPS=1e-8):
        super(PairwiseNegSDR, self).__init__()
        assert sdr_type in ["snr", "sisdr", "sdsdr"]
        self.sdr_type = sdr_type
        self.zero_mean = zero_mean
        self.take_log = take_log
        self.EPS = EPS

    def forward(self, est_targets, targets, length):
        if targets.size() != est_targets.size() or targets.ndim != 3:
            raise TypeError(
                f"Inputs must be of shape [batch, n_src, time], got {targets.size()} and {est_targets.size()} instead"
            )
        assert targets.size() == est_targets.size()
        length = length.to(device)
        mask = length_mask(length).to(device)
        # Step 1. Zero-mean norm
        if self.zero_mean:
            mean_source = torch.sum(targets, dim=2, keepdim=True) / length.view(-1, 1, 1)
            mean_estimate = torch.sum(est_targets, dim=2, keepdim=True) / length.view(-1, 1, 1)
            targets = (targets - mean_source) * torch.unsqueeze(mask, 1)
            est_targets = (est_targets - mean_estimate) * torch.unsqueeze(mask, 1)
        # Step 2. Pair-wise SI-SDR. (Reshape to use broadcast)
        s_target = torch.unsqueeze(targets, dim=1)
        s_estimate = torch.unsqueeze(est_targets, dim=2)

        if self.sdr_type in ["sisdr", "sdsdr"]:
            # [batch, n_src, n_src, 1]
            pair_wise_dot = torch.sum(s_estimate * s_target, dim=3, keepdim=True)
            # [batch, 1, n_src, 1]
            s_target_energy = torch.sum(s_target ** 2, dim=3, keepdim=True) + self.EPS
            # [batch, n_src, n_src, time]
            pair_wise_proj = pair_wise_dot * s_target / s_target_energy
        else:
            # [batch, n_src, n_src, time]
            pair_wise_proj = s_target.repeat(1, s_target.shape[2], 1, 1)
        if self.sdr_type in ["sdsdr", "snr"]:
            e_noise = s_estimate - s_target
        else:
            e_noise = s_estimate - pair_wise_proj
        # [batch, n_src, n_src]
        pair_wise_sdr = torch.sum(pair_wise_proj ** 2, dim=3) / (
            torch.sum(e_noise ** 2, dim=3) + self.EPS
        )
        if self.take_log:
            pair_wise_sdr = 10 * torch.log10(pair_wise_sdr + self.EPS)
        return -pair_wise_sdr

def match_wave_length(x, length):
    if x.size(0) == length:
        return x
    elif x.size(0) > length:
        new_x = x[:length]
        return new_x
    else:
        new_x = torch.zeros(length).to(x.device)
        new_x[:x.size(0)] = x
        return new_x

def length_mask(length):
    mask = torch.zeros(len(length), max(length)).to(device)
    for i in range(len(length)):
        mask[i, :length[i]] = 1
    return mask



# -*- coding: utf-8 -*- #
"""*********************************************************************************************"""
#   FileName     [ pretrain/spec_augment/pretrain_expert.py ]
#   Synopsis     [ the spec augment transformer pretrain expert ]
#   Author       [ Andy T. Liu (https://github.com/andi611) ]
#   Copyright    [ Copyleft(c), Speech Lab, NTU, Taiwan ]
"""*********************************************************************************************"""


###############
# IMPORTATION #
###############
from torch.utils.data import DataLoader
#-------------#
from pretrain.mockingjay.pretrain_expert import UpstreamPretrainExpert as MockingjayPretrainExpert
from pretrain.spec_augment.dataset import KaldiAcousticDataset, OnlineAcousticDataset


####################
# UPSTREAM WRAPPER #
####################
class UpstreamPretrainExpert(MockingjayPretrainExpert):
    """
    The spec augment transformer pretrain expert
    """

    def __init__(self, datarc, upstream_config, device='cuda', multi_gpu=False, **kwargs):
        super(UpstreamPretrainExpert, self).__init__(datarc, upstream_config, device, multi_gpu, **kwargs)

    def _get_train_dataloader(self, extracter):
        if 'libri_root' in self.datarc and 'kaldi' not in self.upstream_config['audio']:
            dataset = OnlineAcousticDataset(extracter,
                                            self.upstream_config['task'],
                                            self.datarc['train_batch_size'],
                                            target_level=self.upstream_config['audio']['target_level'],
                                            **self.datarc)
        else:
            dataset = KaldiAcousticDataset(extracter,
                                           self.upstream_config['task'],
                                           self.datarc['train_batch_size'],
                                           **self.datarc)
        self.dataloader = DataLoader(dataset, batch_size=1, # for bucketing
                                     shuffle=True, num_workers=self.datarc['num_workers'],
                                     drop_last=False, pin_memory=True, collate_fn=dataset.collate_fn)



import re
from pathlib import Path

from torch.utils.data.dataset import Dataset
from torchaudio.sox_effects import apply_effects_file


class QUESST14Dataset(Dataset):
    """QUESST 2014 dataset (English-only)."""

    def __init__(self, split, **kwargs):
        dataset_root = Path(kwargs["dataset_root"])
        doc_paths = english_audio_paths(dataset_root, "language_key_utterances.lst")
        query_paths = english_audio_paths(dataset_root, f"language_key_{split}.lst")

        self.dataset_root = dataset_root
        self.n_queries = len(query_paths)
        self.n_docs = len(doc_paths)
        self.data = query_paths + doc_paths

    def __len__(self):
        return len(self.data)

    def __getitem__(self, idx):
        audio_path = self.data[idx]
        wav, _ = apply_effects_file(
            str(audio_path),
            [
                ["channels", "1"],
                ["rate", "16000"],
                ["gain", "-3.0"],
            ],
        )
        wav = wav.squeeze(0)
        return wav.numpy(), audio_path.with_suffix("").name

    def collate_fn(self, samples):
        """Collate a mini-batch of data."""
        wavs, audio_names = zip(*samples)
        return wavs, audio_names


def english_audio_paths(dataset_root_path, lst_name):
    """Extract English audio paths."""
    audio_paths = []

    with open(dataset_root_path / "scoring" / lst_name) as f:
        for line in f:
            audio_path, lang = tuple(line.strip().split())
            if lang != "nnenglish":
                continue
            audio_path = re.sub(r"^.*?\/", "", audio_path)
            audio_paths.append(dataset_root_path / audio_path)

    return audio_paths



# -*- coding: utf-8 -*- #
"""*********************************************************************************************"""
#   FileName     [ run_pretrain.py ]
#   Synopsis     [ scripts for running the pre-training of upstream models ]
#   Author       [ Andy T. Liu (Andi611) ]
#   Copyright    [ Copyleft(c), Speech Lab, NTU, Taiwan ]
"""*********************************************************************************************"""


###############
# IMPORTATION #
###############
import os
import re
import yaml
import glob
import random
import argparse
import importlib
from shutil import copyfile
from argparse import Namespace
#-------------#
import torch
import numpy as np
#-------------#
from pretrain.runner import Runner
from utility.helper import override


######################
# PRETRAIN ARGUMENTS #
######################
def get_pretrain_args():
    parser = argparse.ArgumentParser()

    # use a ckpt as the experiment initialization
    # if set, all the following args and config will be overwrited by the ckpt, except args.mode
    parser.add_argument('-e', '--past_exp', metavar='{CKPT_PATH,CKPT_DIR}', help='Resume training from a checkpoint')
    parser.add_argument('-o', '--override', help='Used to override args and config, this is at the highest priority')

    # configuration for the experiment, including runner and downstream
    parser.add_argument('-c', '--config', metavar='CONFIG_PATH', help='The yaml file for configuring the whole experiment, except the upstream model')

    # upstream settings
    parser.add_argument('-u', '--upstream', choices=os.listdir('pretrain/'))
    parser.add_argument('-g', '--upstream_config', metavar='CONFIG_PATH', help='The yaml file for configuring the upstream model')

    # experiment directory, choose one to specify
    # expname uses the default root directory: result/pretrain
    parser.add_argument('-n', '--expname', help='Save experiment at expdir/expname')
    parser.add_argument('-p', '--expdir', help='Save experiment at expdir')
    parser.add_argument('-a', '--auto_resume', action='store_true', help='Auto-resume if the expdir contains checkpoints')

    # options
    parser.add_argument('--seed', default=1337, type=int)
    parser.add_argument('--device', default='cuda', help='model.to(device)')
    parser.add_argument('--multi_gpu', action='store_true', help='Enables multi-GPU training')

    args = parser.parse_args()

    if args.expdir is None:
        args.expdir = f'result/pretrain/{args.expname}'

    if args.auto_resume:
        if os.path.isdir(args.expdir):
            ckpt_pths = glob.glob(f'{args.expdir}/states-*.ckpt')
            if len(ckpt_pths) > 0:
                args.past_exp = args.expdir

    if args.past_exp:
        # determine checkpoint path
        if os.path.isdir(args.past_exp):
            ckpt_pths = glob.glob(f'{args.past_exp}/states-*.ckpt')
            assert len(ckpt_pths) > 0
            ckpt_pths = sorted(ckpt_pths, key=lambda pth: int(pth.split('-')[-1].split('.')[0]))
            ckpt_pth = ckpt_pths[-1]
        else:
            ckpt_pth = args.past_exp
        
        print(f'[Runner] - Resume from {ckpt_pth}')

        # load checkpoint
        ckpt = torch.load(ckpt_pth, map_location='cpu')

        def update_args(old, new):
            old_dict = vars(old)
            new_dict = vars(new)
            old_dict.update(new_dict)
            return Namespace(**old_dict)

        # overwrite args and config
        args = update_args(args, ckpt['Args'])
        os.makedirs(args.expdir, exist_ok=True)
        args.init_ckpt = ckpt_pth
        config = ckpt['Config']

    else:
        print('[Runner] - Start a new experiment')
        args.init_ckpt = None

        assert args.expname is not None
        if args.expdir is None:
            args.expdir = f'result/pretrain/{args.expname}'
        os.makedirs(args.expdir, exist_ok=True)

        upstream_dirs = [u for u in os.listdir('pretrain/') if re.search(f'^{u}_|^{u}$', args.upstream)]
        assert len(upstream_dirs) == 1

        if args.config is None:
            args.config = f'pretrain/{upstream_dirs[0]}/config_runner.yaml'
        with open(args.config, 'r') as file:
            config = yaml.load(file, Loader=yaml.FullLoader)
        if os.path.isfile(args.config):
            copyfile(args.config, f'{args.expdir}/config_runner.yaml')
        else:
            raise FileNotFoundError('Wrong file path for runner config.')
        
        if args.upstream_config is None:
            default_upstream_config = f'pretrain/{upstream_dirs[0]}/config_model.yaml'
            assert os.path.isfile(default_upstream_config)
            args.upstream_config = default_upstream_config
        if os.path.isfile(args.upstream_config):
            copyfile(args.upstream_config, f'{args.expdir}/config_model.yaml')
        else:
            raise FileNotFoundError('Wrong file path for model config.')

    if args.override is not None and args.override.lower() != "none":
        override(args.override, args, config)
        os.makedirs(args.expdir, exist_ok=True)

    return args, config


########
# MAIN #
########
def main():
    # get config and arguments
    args, config = get_pretrain_args()

    # Fix seed and make backends deterministic
    random.seed(args.seed)
    np.random.seed(args.seed)
    torch.manual_seed(args.seed)
    if torch.cuda.is_available():
        torch.cuda.manual_seed_all(args.seed)
        torch.backends.cudnn.deterministic = True
        torch.backends.cudnn.benchmark = False

    runner = Runner(args, config)
    eval('runner.train')()
    runner.logger.close()


if __name__ == '__main__':
    main()



"""Downstream expert for Spoken Term Detection on Speech Commands."""

import re
import os
import hashlib
from pathlib import Path
from typing import List, Tuple, Union

import torch
import torch.nn as nn
from torch.utils.data import DataLoader, WeightedRandomSampler
from catalyst.data.sampler import DistributedSamplerWrapper
from torch.distributed import is_initialized
from torch.nn.utils.rnn import pad_sequence

from ..model import *
from .dataset import SpeechCommandsDataset, SpeechCommandsTestingDataset, CLASSES


class DownstreamExpert(nn.Module):
    """
    Used to handle downstream-specific operations
    eg. downstream forward, metric computation, contents to log
    """

    def __init__(self, upstream_dim: int, downstream_expert: dict, expdir: str, **kwargs):
        super(DownstreamExpert, self).__init__()
        self.upstream_dim = upstream_dim
        self.datarc = downstream_expert["datarc"]
        self.modelrc = downstream_expert["modelrc"]

        train_list, valid_list = split_dataset(self.datarc["speech_commands_root"])

        self.train_dataset = SpeechCommandsDataset(train_list, **self.datarc)
        self.dev_dataset = SpeechCommandsDataset(valid_list, **self.datarc)
        self.test_dataset = SpeechCommandsTestingDataset(**self.datarc)

        model_cls = eval(self.modelrc['select'])
        model_conf = self.modelrc.get(self.modelrc['select'], {})
        self.projector = nn.Linear(upstream_dim, self.modelrc['projector_dim'])
        self.model = model_cls(
            input_dim = self.modelrc['projector_dim'],
            output_dim = self.train_dataset.class_num,
            **model_conf,
        )

        self.objective = nn.CrossEntropyLoss()
        self.expdir = expdir
        self.register_buffer('best_score', torch.zeros(1))

    def _get_balanced_train_dataloader(self, dataset, drop_last=False):
        sampler = WeightedRandomSampler(dataset.sample_weights, len(dataset.sample_weights))
        if is_initialized():
            sampler = DistributedSamplerWrapper(sampler)
        return DataLoader(
            dataset,
            sampler=sampler,
            batch_size=self.datarc["batch_size"],
            drop_last=drop_last,
            num_workers=self.datarc["num_workers"],
            collate_fn=dataset.collate_fn,
        )

    def _get_balanced_dev_dataloader(self, dataset, drop_last=False):
        return DataLoader(
            dataset,
            sampler=WeightedRandomSampler(
                dataset.sample_weights, len(dataset.sample_weights)
            ),
            batch_size=self.datarc["batch_size"],
            drop_last=drop_last,
            num_workers=self.datarc["num_workers"],
            collate_fn=dataset.collate_fn,
        )

    def _get_dataloader(self, dataset):
        return DataLoader(
            dataset,
            shuffle=False,
            batch_size=self.datarc["batch_size"],
            drop_last=False,
            num_workers=self.datarc["num_workers"],
            collate_fn=dataset.collate_fn,
        )

    # Interface
    def get_dataloader(self, mode):
        if mode == 'train':
            return self._get_balanced_train_dataloader(self.train_dataset, drop_last=True)
        elif mode == 'dev':
            return self._get_balanced_dev_dataloader(self.dev_dataset, drop_last=False)
        elif mode == 'test':
            return self._get_dataloader(self.test_dataset)
        else:
            raise NotImplementedError

    # Interface
    def forward(self, mode, features, labels, filenames, records, **kwargs):
        device = features[0].device
        features_len = torch.IntTensor([len(feat) for feat in features]).to(device=device)
        features = pad_sequence(features, batch_first=True)
        features = self.projector(features)
        predicted, _ = self.model(features, features_len)

        labels = torch.LongTensor(labels).to(features.device)
        loss = self.objective(predicted, labels)

        predicted_classid = predicted.max(dim=-1).indices
        records["loss"].append(loss.item())
        records["acc"] += (predicted_classid == labels).view(-1).cpu().float().tolist()

        records["filename"] += filenames
        records["predict"] += [CLASSES[idx] for idx in predicted_classid.cpu().tolist()]
        records["truth"] += [CLASSES[idx] for idx in labels.cpu().tolist()]

        return loss

    # interface
    def log_records(self, mode, records, logger, global_step, **kwargs):
        save_names = []
        for key in ["loss", "acc"]:
            values = records[key]
            average = sum(values) / len(values)
            logger.add_scalar(
                f'speech_commands/{mode}-{key}',
                average,
                global_step=global_step
            )
            with open(Path(self.expdir, "log.log"), 'a') as f:
                if key == 'acc':
                    print(f"{mode} {key}: {average}")
                    f.write(f'{mode} at step {global_step}: {average}\n')
                    if mode == 'dev' and average > self.best_score:
                        self.best_score = torch.ones(1) * average
                        f.write(f'New best on {mode} at step {global_step}: {average}\n')
                        save_names.append(f'{mode}-best.ckpt')

        with open(Path(self.expdir) / f"{mode}_predict.txt", "w") as file:
            lines = [f"{f} {i}\n" for f, i in zip(records["filename"], records["predict"])]
            file.writelines(lines)

        with open(Path(self.expdir) / f"{mode}_truth.txt", "w") as file:
            lines = [f"{f} {i}\n" for f, i in zip(records["filename"], records["truth"])]
            file.writelines(lines)

        return save_names


def split_dataset(
    root_dir: Union[str, Path], max_uttr_per_class=2 ** 27 - 1
) -> Tuple[List[Tuple[str, str]], List[Tuple[str, str]]]:
    """Split Speech Commands into 3 set.
    
    Args:
        root_dir: speech commands dataset root dir
        max_uttr_per_class: predefined value in the original paper
    
    Return:
        train_list: [(class_name, audio_path), ...]
        valid_list: as above
    """
    train_list, valid_list = [], []

    for entry in Path(root_dir).iterdir():
        if not entry.is_dir() or entry.name == "_background_noise_":
            continue

        for audio_path in entry.glob("*.wav"):
            speaker_hashed = re.sub(r"_nohash_.*$", "", audio_path.name)
            hashed_again = hashlib.sha1(speaker_hashed.encode("utf-8")).hexdigest()
            percentage_hash = (int(hashed_again, 16) % (max_uttr_per_class + 1)) * (
                100.0 / max_uttr_per_class
            )

            if percentage_hash < 10:
                valid_list.append((entry.name, audio_path))
            elif percentage_hash < 20:
                pass  # testing set is discarded
            else:
                train_list.append((entry.name, audio_path))

    return train_list, valid_list



# -*- coding: utf-8 -*- #
"""*********************************************************************************************"""
#   FileName     [ dataset.py ]
#   Synopsis     [ The VCC2020 dataset ]
#   Author       [ Wen-Chin Huang (https://github.com/unilight) ]
#   Copyright    [ Copyright(c), Toda Lab, Nagoya University, Japan ]
"""*********************************************************************************************"""


import os
import random

import librosa
import numpy as np

import torch
import torch.nn as nn
from torch.nn.utils.rnn import pad_sequence
from torch.utils.data.dataset import Dataset

import torchaudio
from .utils import logmelspectrogram

SRCSPKS = ["SEF1", "SEF2", "SEM1", "SEM2"]
FS = 16000 # Always resample to 16kHz

class VCC2020Dataset(Dataset):
    def __init__(self, split, trgspk, data_root, lists_root, fbank_config, train_dev_seed=1337, **kwargs):
        super(VCC2020Dataset, self).__init__()

        self.trgspk = trgspk
        self.trg_lang = trgspk[1]
        self.fbank_config = fbank_config

        X = []
        if split == 'train' or split == 'dev':
            file_list = open(os.path.join(lists_root, self.trg_lang + "_" + split + '_list.txt')).read().splitlines()
            for number in file_list:
                wav_path = os.path.join(data_root, trgspk, number + ".wav")
                if os.path.isfile(wav_path):
                    X.append(wav_path)
            random.seed(train_dev_seed)
            random.shuffle(X)
        elif split == 'test':
            file_list = open(os.path.join(lists_root, 'eval_list.txt')).read().splitlines()
            X = [os.path.join(data_root, srcspk, number + ".wav") for number in file_list for srcspk in SRCSPKS]
        else:
            raise ValueError('Invalid \'split\' argument for dataset: VCC2020Dataset!')
        print('[Dataset] - number of data for ' + split + ': ' + str(len(X)))
        self.X = X

    def _load_wav(self, wav_path, fs):
        # use librosa to resample. librosa gives range [-1, 1]
        wav, sr = librosa.load(wav_path, sr=fs)
        return wav, sr

    def __len__(self):
        return len(self.X)

    def __getitem__(self, index):
        wav_path = self.X[index]
        wav_original, fs_original = self._load_wav(wav_path, fs=None)
        wav_resample, fs_resample = self._load_wav(wav_path, fs=FS)

        lmspc = logmelspectrogram(
            x=wav_original,
            fs=fs_original,
            n_mels=self.fbank_config["n_mels"],
            n_fft=self.fbank_config["n_fft"],
            n_shift=self.fbank_config["n_shift"],
            win_length=self.fbank_config["win_length"],
            window=self.fbank_config["window"],
            fmin=self.fbank_config["fmin"],
            fmax=self.fbank_config["fmax"],
        )

        return wav_resample, wav_original, lmspc, wav_path
    
    def collate_fn(self, batch):
        sorted_batch = sorted(batch, key=lambda x: -x[1].shape[0])
        bs = len(sorted_batch) # batch_size
        wavs = [torch.from_numpy(sorted_batch[i][0]) for i in range(bs)]
        wavs_2 = [torch.from_numpy(sorted_batch[i][1]) for i in range(bs)] # This is used for obj eval
        acoustic_features = [torch.from_numpy(sorted_batch[i][2]) for i in range(bs)]
        acoustic_features_padded = pad_sequence(acoustic_features, batch_first=True)
        acoustic_feature_lengths = torch.from_numpy(np.array([acoustic_feature.size(0) for acoustic_feature in acoustic_features]))
        wav_paths = [sorted_batch[i][3] for i in range(bs)]
        
        return wavs, wavs_2, acoustic_features, acoustic_features_padded, acoustic_feature_lengths, wav_paths


class CustomDataset(Dataset):
    def __init__(self, eval_list_file, **kwargs):
        super(CustomDataset, self).__init__()

        X = []
        if os.path.isfile(eval_list_file):
            print("[Dataset] Reading custom eval list file: {}".format(eval_list_file))
            X = open(eval_list_file, "r").read().splitlines()
        else:
            raise ValueError("[Dataset] eval list file does not exist: {}".format(eval_list_file))
        print('[Dataset] - number of data for custom test: ' + str(len(X)))
        self.X = X

    def _load_wav(self, wav_path, fs):
        # use librosa to resample. librosa gives range [-1, 1]
        wav, sr = librosa.load(wav_path, sr=fs)
        return wav, sr

    def __len__(self):
        return len(self.X)

    def __getitem__(self, index):
        wav_path = self.X[index]
        wav_original, fs_original = self._load_wav(wav_path, fs=None)
        wav_resample, fs_resample = self._load_wav(wav_path, fs=FS)

        return wav_resample, wav_original, wav_path
    
    def collate_fn(self, batch):
        sorted_batch = sorted(batch, key=lambda x: -x[1].shape[0])
        bs = len(sorted_batch) # batch_size
        wavs = [torch.from_numpy(sorted_batch[i][0]) for i in range(bs)]
        wavs_2 = [torch.from_numpy(sorted_batch[i][1]) for i in range(bs)] # This is used for obj eval
        wav_paths = [sorted_batch[i][2] for i in range(bs)]
        
        return wavs, wavs_2, None, None, None, wav_paths